'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.createStore = exports.actions = undefined;

var _createStore2 = require('redux/lib/createStore');

var _createStore3 = _interopRequireDefault(_createStore2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * state shape
 *
 * {
 *   items: [
 *     {id, args, page(args), scrollTop, lastActivatedAt, isActive, cacheTime, cacheTimer}
 *   ]
 * }
 *
 */

var INIT_STATE = {
    items: []
};

// actions

var SET_ITEM = 'SET_ITEM';
var REMOVE_ITEM = 'REMOVE_ITEM';
var SET_ITEMS = 'SET_ITEMS';

// action creators

var setItem = function setItem(item) {
    return { type: SET_ITEM, item: item };
};

var removeItem = function removeItem(id) {
    return { type: REMOVE_ITEM, id: id };
};

var setItems = function setItems(items) {
    return { type: SET_ITEMS, items: items };
};

var actions = {
    setItem: setItem,
    removeItem: removeItem,
    setItems: setItems
};

// reducer

var items = function items() {
    var items = arguments.length <= 0 || arguments[0] === undefined ? [] : arguments[0];
    var action = arguments[1];

    switch (action.type) {
        case SET_ITEM:
            return items.filter(function (item) {
                return item.id !== action.item.id;
            }).concat(action.item);
        case REMOVE_ITEM:
            return items.filter(function (item) {
                return item.id !== action.id;
            });
        case SET_ITEMS:
            return action.items;
        default:
            return items;
    }
};

var reducer = function reducer() {
    var state = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
    var action = arguments[1];

    return {
        items: items(state.items, action)
    };
};

// store

function createStore() {
    return (0, _createStore3.default)(reducer, INIT_STATE);
}

// export

exports.actions = actions;
exports.createStore = createStore;